@@ -20,27 +20,35 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-__all__ = ['GreenSocket', 'GreenFile', 'GreenPipe']
+"""Cooperative socket module.
+
+This module provides socket operations and some related functions.
+The API of the functions and classes matches the API of the corresponding
+items in standard :mod:`socket` module exactly, but the synchronous functions
+in this module only block the current greenlet and let the others run.
+"""
+
+__all__ = ['socket', 'socketpair', 'fromfd', 'gethostbyname', 'getaddrinfo', 'getnameinfo', 'wrap_ssl']
 
 import _socket
-_original_socket = _socket.socket
-_original_fromfd = _socket.fromfd
-_original_socketpair = _socket.socketpair
 error = _socket.error
 timeout = _socket.timeout
-getaddrinfo = _socket.getaddrinfo # XXX implement this and others using libevent's dns
 __socket__ = __import__('socket')
 _fileobject = __socket__._fileobject
 sslerror = __socket__.sslerror
+gaierror = __socket__.gaierror
 
 import sys
 import errno
 import time
+import random
+import re
 
-from gevent.hub import getcurrent, get_hub, spawn_raw
+from gevent.hub import getcurrent, get_hub, spawn_raw, Waiter
 from gevent import core
 
 BUFFER_SIZE = 4096
+_ip_re = re.compile('^[\d\.]+$')
 
 
 def _wait_helper(ev, evtype):
@@ -51,20 +59,20 @@ def _wait_helper(ev, evtype):
         current.switch(ev)
 
 
-def wait_reader(fileno, timeout=-1, timeout_exc=_socket.timeout):
+def wait_read(fileno, timeout=-1, timeout_exc=_socket.timeout):
     evt = core.read_event(fileno, _wait_helper, timeout, (getcurrent(), timeout_exc))
     try:
         switch_result = get_hub().switch()
-        assert evt is switch_result, 'Invalid switch into wait_reader(): %r' % (switch_result, )
+        assert evt is switch_result, 'Invalid switch into wait_read(): %r' % (switch_result, )
     finally:
         evt.cancel()
 
 
-def wait_writer(fileno, timeout=-1, timeout_exc=_socket.timeout):
+def wait_write(fileno, timeout=-1, timeout_exc=_socket.timeout):
     evt = core.write_event(fileno, _wait_helper, timeout, (getcurrent(), timeout_exc))
     try:
         switch_result = get_hub().switch()
-        assert evt is switch_result, 'Invalid switch into wait_writer(): %r' % (switch_result, )
+        assert evt is switch_result, 'Invalid switch into wait_write(): %r' % (switch_result, )
     finally:
         evt.cancel()
 
@@ -136,12 +144,12 @@ _delegate_methods = ("recv", "recvfrom", "recv_into", "recvfrom_into", "send", "
 
 timeout_default = object()
 
-class GreenSocket(object):
+class socket(object):
     is_secure = False # XXX remove this
 
     def __init__(self, family_or_realsock=_socket.AF_INET, *args, **kwargs):
         if isinstance(family_or_realsock, (int, long)):
-            self.fd = _original_socket(family_or_realsock, *args, **kwargs)
+            self.fd = _socket.socket(family_or_realsock, *args, **kwargs)
             self.timeout = _socket.getdefaulttimeout()
         else:
             if hasattr(family_or_realsock, '_sock'):
@@ -201,7 +209,7 @@ class GreenSocket(object):
             if res is not None:
                 client, addr = res
                 return type(self)(client), addr
-            wait_reader(fd.fileno(), timeout=self.timeout, timeout_exc=timeout)
+            wait_read(fd.fileno(), timeout=self.timeout, timeout_exc=timeout)
 
     def close(self):
         self.fd = _closedsocket()
@@ -210,12 +218,14 @@ class GreenSocket(object):
             setattr(self, method, dummy)
 
     def connect(self, address):
+        if isinstance(address, tuple) and len(address)==2:
+            address = gethostbyname(address[0]), address[1]
         if self.timeout==0.0:
             return self.fd.connect(address)
         fd = self.fd
         if self.timeout is None:
             while not socket_connect(fd, address):
-                wait_writer(fd.fileno(), timeout_exc=timeout)
+                wait_write(fd.fileno(), timeout_exc=timeout)
         else:
             end = time.time() + self.timeout
             while True:
@@ -223,7 +233,7 @@ class GreenSocket(object):
                     return
                 if time.time() >= end:
                     raise timeout
-                wait_writer(fd.fileno(), timeout=end-time.time(), timeout_exc=timeout)
+                wait_write(fd.fileno(), timeout=end-time.time(), timeout_exc=timeout)
 
     def connect_ex(self, address):
         if self.timeout==0.0:
@@ -232,7 +242,7 @@ class GreenSocket(object):
         if self.timeout is None:
             while not socket_connect(fd, address):
                 try:
-                    wait_writer(fd.fileno(), timeout_exc=timeout)
+                    wait_write(fd.fileno(), timeout_exc=timeout)
                 except error, ex:
                     return ex[0]
         else:
@@ -243,7 +253,7 @@ class GreenSocket(object):
                 if time.time() >= end:
                     raise timeout
                 try:
-                    wait_writer(fd.fileno(), timeout=end-time.time(), timeout_exc=timeout)
+                    wait_write(fd.fileno(), timeout=end-time.time(), timeout_exc=timeout)
                 except error, ex:
                     return ex[0]
 
@@ -258,30 +268,30 @@ class GreenSocket(object):
 
     def recv(self, *args):
         if self.timeout!=0.0:
-            wait_reader(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
+            wait_read(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
         res = self.fd.recv(*args)
         return res
 
     def recvfrom(self, *args):
         if self.timeout!=0.0:
-            wait_reader(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
+            wait_read(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
         return self.fd.recvfrom(*args)
 
     def recvfrom_into(self, *args):
         if self.timeout!=0.0:
-            wait_reader(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
+            wait_read(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
         return self.fd.recvfrom_into(*args)
 
     def recv_into(self, *args):
         if self.timeout!=0.0:
-            wait_reader(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
+            wait_read(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
         return self.fd.recv_into(*args)
 
     def send(self, data, timeout=timeout_default):
         if timeout is timeout_default:
             timeout = self.timeout
         if timeout!=0.0:
-            wait_writer(self.fileno(), timeout=timeout, timeout_exc=_socket.timeout)
+            wait_write(self.fileno(), timeout=timeout, timeout_exc=_socket.timeout)
         return self.fd.send(data)
 
     def sendall(self, data):
@@ -303,7 +313,7 @@ class GreenSocket(object):
 
     def sendto(self, *args):
         if self.timeout!=0.0:
-            wait_writer(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
+            wait_write(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
         return self.fd.sendto(*args)
 
     def setblocking(self, flag):
@@ -328,15 +338,16 @@ class GreenSocket(object):
     def gettimeout(self):
         return self.timeout
 
+GreenSocket = socket # XXX this alias will be removed
 
 SysCallError_code_mapping = {-1: 8}
 
 
-class GreenSSL(GreenSocket):
+class GreenSSL(socket):
     is_secure = True
 
     def __init__(self, fd, server_side=False):
-        GreenSocket.__init__(self, fd)
+        socket.__init__(self, fd)
         self._makefile_refs = 0
         if server_side:
             self.fd.set_accept_state()
@@ -367,7 +378,7 @@ class GreenSSL(GreenSocket):
                 accepted = type(self)(client, server_side=True)
                 accepted.do_handshake()
                 return accepted, addr
-            wait_reader(fd.fileno(), timeout=self.timeout, timeout_exc=timeout)
+            wait_read(fd.fileno(), timeout=self.timeout, timeout_exc=timeout)
 
     def do_handshake(self):
         while True:
@@ -375,16 +386,16 @@ class GreenSSL(GreenSocket):
                 self.fd.do_handshake()
                 break
             except SSL.WantReadError:
-                wait_reader(self.fileno())
+                wait_read(self.fileno())
             except SSL.WantWriteError:
-                wait_writer(self.fileno())
+                wait_write(self.fileno())
             except SSL.SysCallError, ex:
                 raise sslerror(SysCallError_code_mapping.get(ex.args[0], ex.args[0]), ex.args[1])
             except SSL.Error, ex:
                 raise sslerror(str(ex))
 
     def connect(self, *args):
-        GreenSocket.connect(self, *args)
+        socket.connect(self, *args)
         self.do_handshake()
 
     def send(self, data, timeout=timeout_default):
@@ -397,12 +408,12 @@ class GreenSSL(GreenSocket):
                 if self.timeout==0.0:
                     raise timeout(str(ex))
                 else:
-                    wait_writer(self.fileno(), timeout=timeout, timeout_exc=_socket.timeout)
+                    wait_write(self.fileno(), timeout=timeout, timeout_exc=_socket.timeout)
             except SSL.WantReadError, ex:
                 if self.timeout==0.0:
                     raise timeout(str(ex))
                 else:
-                    wait_reader(self.fileno(), timeout=timeout, timeout_exc=_socket.timeout)
+                    wait_read(self.fileno(), timeout=timeout, timeout_exc=_socket.timeout)
             except SSL.SysCallError, e:
                 if e[0] == -1 and data == "":
                     # errors when writing empty strings are expected and can be ignored
@@ -422,12 +433,12 @@ class GreenSSL(GreenSocket):
                 if self.timeout==0.0:
                     raise timeout(str(ex))
                 else:
-                    wait_reader(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
+                    wait_read(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
             except SSL.WantWriteError, ex:
                 if self.timeout==0.0:
                     raise timeout(str(ex))
                 else:
-                    wait_reader(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
+                    wait_read(self.fileno(), timeout=self.timeout, timeout_exc=timeout)
             except SSL.ZeroReturnError:
                 return ''
             except SSL.SysCallError, ex:
@@ -461,12 +472,12 @@ class GreenSSL(GreenSocket):
 
 
 def socketpair(*args):
-    one, two = _original_socketpair(*args)
-    return GreenSocket(one), GreenSocket(two)
+    one, two = _socket.socketpair(*args)
+    return socket(one), socket(two)
 
 
 def fromfd(*args):
-    return GreenSocket(_original_fromfd(*args))
+    return socket(_socket.fromfd(*args))
 
 def socket_bind_and_listen(descriptor, addr=('', 0), backlog=50):
     set_reuse_addr(descriptor)
@@ -490,7 +501,7 @@ def tcp_listener(address, backlog=50):
     which accepts connections forever and spawns greenlets for
     each incoming connection.
     """
-    sock = GreenSocket()
+    sock = socket()
     socket_bind_and_listen(sock, address, backlog=backlog)
     return sock
 
@@ -508,7 +519,7 @@ def ssl_listener(address, private_key, certificate):
     which accepts connections forever and spawns greenlets for
     each incoming connection.
     """
-    r = _original_socket()
+    r = _socket.socket()
     sock = wrap_ssl000(r, private_key, certificate)
     socket_bind_and_listen(sock, address)
     return sock
@@ -519,7 +530,7 @@ def connect_tcp(address, localaddr=None):
     Create a TCP connection to address (host, port) and return the socket.
     Optionally, bind to localaddr (host, port) first.
     """
-    desc = GreenSocket()
+    desc = socket()
     if localaddr is not None:
         desc.bind(localaddr)
     desc.connect(address)
@@ -571,7 +582,7 @@ def create_connection(address, timeout=_GLOBAL_DEFAULT_TIMEOUT):
         af, socktype, proto, canonname, sa = res
         sock = None
         try:
-            sock = GreenSocket(af, socktype, proto)
+            sock = socket(af, socktype, proto)
             if timeout is not _GLOBAL_DEFAULT_TIMEOUT:
                 sock.settimeout(timeout)
             sock.connect(sa)
@@ -599,7 +610,7 @@ def create_connection_ssl(address, timeout=_GLOBAL_DEFAULT_TIMEOUT):
         af, socktype, proto, canonname, sa = res
         sock = None
         try:
-            _sock = _original_socket(af, socktype, proto)
+            _sock = _socket.socket(af, socktype, proto)
             sock = wrap_ssl000(_sock)
             if timeout is not _GLOBAL_DEFAULT_TIMEOUT:
                 sock.settimeout(timeout)
@@ -635,6 +646,113 @@ def wrap_ssl000(sock, keyfile=None, certfile=None):
 
     return ssl_sock
 
+try:
+    core.dns_init()
+except:
+    # fallback to blocking versions
+    gethostbyname = __socket__.gethostbyname
+    getaddrinfo = __socket__.getaddrinfo
+    getnameinfo = __socket__.getnameinfo
+else:
+    # NOTE:
+    # use flags=core.DNS_QUERY_NO_SEARCH to avoid search, see comments in evdns.h
+    # TODO:
+    # might need to map evdns errors to socket errors
+    # for example, DNS_ERR_NOTEXIST(3) is:
+    # socket.gaierror: [Errno -2] Name or service not known
+
+    def _dns_helper(result, type, ttl, addrs, args):
+        (waiter,) = args
+        waiter.switch((result, type, ttl, addrs))
+
+    def gethostbyname(hostname):
+        """gethostbyname implemented using EvDNS.
+
+        Differs in the following ways:
+
+        * raises gaierror with EvDNS error codes instead of standard socket error codes
+        * does not support /etc/hosts (see code for hacks to make localhost work)
+        * does not iterate through all addresses, instead picks a random one each time
+        """
+        # TODO: this is supposed to iterate through all the addresses
+        # could use a global dict(hostname, iter)
+        # - fix these nasty hacks for localhost, ips, etc.
+        if hostname == 'localhost': # QQQ should use /etc/hosts
+            return '127.0.0.1'
+        if _ip_re.match(hostname):
+            return hostname
+        if hostname == _socket.gethostname():
+            return _socket.gethostbyname(hostname)
+        waiter = Waiter()
+        core.dns_resolve_ipv4(hostname, 0, _dns_helper, waiter)
+        result, type, ttl, addrs = waiter.wait()
+        if result != core.DNS_ERR_NONE:
+            raise gaierror('dns_resolve_ipv4 returned %s' % result)
+        return random.choice(addrs)
+
+    def getaddrinfo(host, port, family=__socket__.AF_UNSPEC, socktype=__socket__.SOCK_STREAM, proto=0, flags=0):
+        """getaddrinfo implemented using EvDNS.
+
+        Differs in the following ways:
+
+        * raises gaierror with EvDNS error codes instead of standard socket error codes
+        * does not support /etc/hosts
+        * IPv6 support is untested.
+        * AF_UNSPEC only tries IPv4
+        * only supports TCP, UDP, IP protocols
+        * port must be numeric, does not support string service names. see socket.getservbyname
+        * only supported value for flags is core.DNS_QUERY_NO_SEARCH, see evdns.h
+        """
+        if _ip_re.match(host):
+            return [(__socket__.AF_INET, socktype, p, '', (host, port)) for p in (6, 17, 0)]
+        waiter = Waiter()
+        if family == __socket__.AF_INET:
+            core.dns_resolve_ipv4(host, flags, _dns_helper, waiter)
+        elif family == __socket__.AF_INET6:
+            core.dns_resolve_ipv6(host, flags, _dns_helper, waiter)
+        elif family == __socket__.AF_UNSPEC:
+            # TODO: AF_UNSPEC means try both AF_INET and AF_INET6
+            family = __socket__.AF_INET
+            core.dns_resolve_ipv4(host, flags, _dns_helper, waiter)
+        else:
+            raise NotImplementedError
+        result, type, ttl, addrs = waiter.wait()
+        if result != core.DNS_ERR_NONE:
+            raise gaierror(result)
+        r = []
+        for addr in addrs:
+            for p in (6, 17, 0): # tcp, udp, ip protocols
+                r.append((family, socktype, p, '', (addr, port)))
+        return r
+
+    def getnameinfo(sockaddr, flags):
+        """getnameinfo implemented using EvDNS.
+
+        Differs in the following ways:
+
+        * raises gaierror with EvDNS error codes instead of standard socket error codes
+        * does not support /etc/hosts
+        * IPv6 support is untested.
+        * port must be numeric, does not support string service names. see socket.getservbyname
+        * only supported value for flags is core.DNS_QUERY_NO_SEARCH, see evdns.h
+        """
+
+        # http://svn.python.org/view/python/trunk/Modules/socketmodule.c?view=markup
+        # see socket_getnameinfo
+        try:
+            host, port = sockaddr[:2]
+            port = int(port)
+        except ValueError:
+            # make testRefCountGetNameInfo pass
+            del sockaddr
+            raise SystemError
+        waiter = Waiter()
+        core.dns_resolve_reverse(host, flags, _dns_helper, waiter)
+        result, type, ttl, addrs = waiter.wait()
+        if result != core.DNS_ERR_NONE:
+            raise gaierror(result)
+        return (addrs, port)
+
 
 def wrap_ssl(sock, keyfile=None, certfile=None):
     from OpenSSL import SSL
